diff -Nur --exclude=CVS dietlibc-orig/Makefile dietlibc/Makefile
--- dietlibc-orig/Makefile	Tue May 13 21:47:00 2003
+++ dietlibc/Makefile	Tue May 13 20:36:39 2003
@@ -394,6 +394,9 @@
 # these depend on dietfeatures.h for WANT_DYNAMIC
 $(OBJDIR)/start.o $(OBJDIR)/dyn_start.o $(OBJDIR)/dyn_stop.o: dietfeatures.h
 
+# these depend on dietfeatures.h for WANT_MULTI_SEED_RNG
+$(OBJDIR)/rand.o $(OBJDIR)/rand_r.o: dietfeatures.h
+
 $(OBJDIR)/unified.o: dietuglyweaks.h
 
 $(OBJDIR)/adjtimex.o: include/sys/timex.h
diff -Nur --exclude=CVS dietlibc-orig/dietfeatures.h dietlibc/dietfeatures.h
--- dietlibc-orig/dietfeatures.h	Tue May 13 21:47:00 2003
+++ dietlibc/dietfeatures.h	Tue May 13 22:48:46 2003
@@ -85,6 +85,12 @@
  * WARNING: this violates C99 */
 /* #define WANT_MALLOC_ZERO */
 
+/* do you want a multiple-seed random number generator as described in
+ * Communication of the ACM, May 2003.
+ */
+  
+/* #define WANT_MULTI_SEED_RNG */
+
 
 /* stop uncommenting here ;-) */
 #ifndef WANT_FASTER_STRING_ROUTINES
@@ -108,5 +114,11 @@
 #undef WANT_SAFEGUARD
 #endif
 #endif
+
+#ifdef WANT_MULTI_SEED_RNG
+#define SEED_SIZE 1024
+#endif
+
+#define RANDOM_DEVICE "/dev/urandom"
 
 #endif
diff -Nur --exclude=CVS dietlibc-orig/include/stdlib.h dietlibc/include/stdlib.h
--- dietlibc-orig/include/stdlib.h	Tue May 13 21:47:04 2003
+++ dietlibc/include/stdlib.h	Tue May 13 20:49:45 2003
@@ -48,6 +48,7 @@
 #ifdef _BSD_SOURCE
 extern int random(void) __THROW;
 extern void srandom(unsigned int seed) __THROW;
+extern void srandomdev(void) __THROW;
 #endif
 
 typedef unsigned short randbuf[3];
diff -Nur --exclude=CVS dietlibc-orig/lib/rand.c dietlibc/lib/rand.c
--- dietlibc-orig/lib/rand.c	Tue Apr  1 18:00:31 2003
+++ dietlibc/lib/rand.c	Tue May 13 22:54:05 2003
@@ -1,12 +1,59 @@
 #include <stdlib.h>
+#include <unistd.h>
+#include <fcntl.h>
+#include <time.h>
 
+#include "dietfeatures.h"
+
+#ifdef WANT_MULTI_SEED_RNG
+static unsigned int seed[SEED_SIZE];
+#else
 static unsigned int seed=1;
+#endif
 
 int rand(void) {
-  return rand_r(&seed);
+  return rand_r(
+#ifdef WANT_MULTI_SEED_RNG
+   seed
+#else
+   &seed
+#endif
+  );
+}
+
+void srand(unsigned int i) { 
+#ifdef WANT_MULTI_SEED_RNG
+  int x;
+  for (x=0;x<SEED_SIZE;x++) {
+    i^=i<<13;
+    i^=i>>17;
+    i^=i<<5;
+    seed[x] = i;
+  }
+#else
+  seed=i; 
+#endif
+}
+
+void srandomdev(void) {
+  int fd = open(RANDOM_DEVICE,O_RDONLY);
+  if (fd != -1 &&
+#ifdef WANT_MULTI_SEED_RNG
+        (read(fd,seed,sizeof(seed)) == sizeof(seed))
+#else
+        (read(fd,&seed,sizeof(seed)) == sizeof(seed))
+#endif
+      ) {
+    close(fd);
+  } else {
+    unsigned int junk;
+    if (fd != -1) {
+      close(fd);
+    }
+    srand(time(NULL)^getpid()^junk); /* fall back (OpenBSD does it similar) */
+  }
 }
 
-void srand(unsigned int i) { seed=i; }
 
 int random(void) __attribute__((alias("rand")));
 void srandom(unsigned int i) __attribute__((alias("srand")));
diff -Nur --exclude=CVS dietlibc-orig/lib/rand_r.c dietlibc/lib/rand_r.c
--- dietlibc-orig/lib/rand_r.c	Tue Apr  1 18:00:31 2003
+++ dietlibc/lib/rand_r.c	Tue May 13 22:47:54 2003
@@ -8,6 +8,7 @@
 
 #include <stdlib.h>
 #include <stdint.h>
+#include "dietfeatures.h"
 
 /* Knuth's TAOCP section 3.6 */
 #define	M	((1U<<31) -1)
@@ -18,13 +19,26 @@
 // FIXME: ISO C/SuS want a longer period
 
 int rand_r(unsigned int* seed)
-{   int32_t X;
+{   
+#ifdef WANT_MULTI_SEED_RNG
+    unsigned long long t, a = 123471786LL;
+    static unsigned int c = 362436, i = (SEED_SIZE-1); /* not exactly reentrant *shrug* */
+    unsigned int x, r = 0xFFFFFFFE;
+    i = (i+1) & 1023;
+    t = a * seed[i] + c;
+    c = (t>>32);
+    x = t + c;
+    if (x<c) { x++; c++; }
+    return (seed[i] = r - x);
+#else
+    int32_t X;
 
     X = *seed;
     X = A*(X%Q) - R * (int32_t) (X/Q);
     if (X < 0)
-	X += M;
+        X += M;
 
     *seed = X;
     return X;
+#endif
 }
